
from typing import TYPE_CHECKING
from .importer import transImporter
from .exporter import transExporter
import os
import argparse
import re
if TYPE_CHECKING:
    from systemrdl.node import AddrmapNode
    from systemrdl import RDLCompiler, RDLCompileError
    import argparse
class MyImporterDescriptor:
    file_extensions = ["xlsx", "csv"]
    short_desc = "give a csv/xlxs file."
    long_desc = "..."

    def is_compatible(self, path: str) -> bool:
        # Could be any XML file.
        # See if file contains an ipxact or spirit component tag
        #with open(path, "r", encoding="utf-8") as f:
        #    if re.search(r"<(spirit|ipxact):component\b", f.read()):
        #        return True
        return True
      
    def add_importer_arguments(self, arg_group: 'argparse.ArgumentParser') -> None:

         arg_group.add_argument("--csv", help="display a path of csv file", type=str)

    def do_import(self,  rdlc: 'RDLCompiler', options: 'argparse.Namespace') -> None:
        print('bbbb')
       


class MyExporterDescriptor:
    short_desc = "transfer csv/xlxs  to rdl."
    def add_exporter_arguments(self, arg_group: 'argparse.ArgumentParser') -> None:
        arg_group.add_argument("-xlsx1", help="display a path of xlsx1 file", type=str)

    def do_export(self,  top_node: 'AddrmapNode',options: 'argparse.Namespace') -> None:
        trans1=transExporter()
        trans1.export_file(options.xlsx1,options.output)
        



from .format_translator import format_translator
from .rkvGensystemRDL import GensystemRDL
import os
from systemrdl import RDLCompiler
class transImporter():

    def __init__(self, compiler: RDLCompiler):
        """
        Parameters
        ----------
        """



    def import_file(self,xlsx_name) -> None:
        """
        Import a single SPIRIT or IP-XACT file into the SystemRDL namespace.

        Parameters
        ----------
        path:
            Input SPIRIT or IP-XACT XML file.
        remap_state:
            Optional remapState string that is used to select memoryRemap regions
            that are tagged under a specific remap state.
        """
      #  format_trans=format_translator()
      #  if not os.path.exists(xlsx_name):
      #      print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
      #      print('ERROR')
      #      exit()
      # #if not os.path.exists(csv_name):
      # #    print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
      # #    print('ERROR')
      # #    exit()
      #  print(xlsx_name)
      #  format_trans.XLSX2CSV(xlsx_name)
        pass


    



from .rkvGensystemRDL import GensystemRDL
from .format_translator import format_translator
import os
from systemrdl import RDLCompiler
class transExporter():

    def __init__(self,):
        """
        Parameters
        ----------
        """

    


    def export_file(self,xlsx_name,rdlname,) -> None:
        """
        Import a single SPIRIT or IP-XACT file into the SystemRDL namespace.

        Parameters
        ----------
        path:
            Input SPIRIT or IP-XACT XML file.
        remap_state:
            Optional remapState string that is used to select memoryRemap regions
            that are tagged under a specific remap state.
        """
        format_trans=format_translator()
        if not os.path.exists(xlsx_name):
            print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
            print('ERROR')
            exit()
       #if not os.path.exists(csv_name):
       #    print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
       #    print('ERROR')
       #    exit()
        print(xlsx_name)
        format_trans.XLSX2CSV(xlsx_name)
        print('%s \n transfer success \n%s' % (40*'*', 40*'*'))
        gen = GensystemRDL()
        print(format_trans.csv_name)
        gen.readRgmFile(format_trans.csv_name)
        #outpath=outdir+rdlname
        gen.gensystemRDLFile(rdlname)
        print('aaa')
        print(rdlname)
        print('%s \ntransfer success \n%s' % (40*'*', 40*'*'))

    



# coding = utf-8
import os
import pandas as pd
import csv
from openpyxl import load_workbook

class format_translator:
    csv_name='test1.csv'
 
    def XLSX2CSV(self,data_filenamepath):
        """
        转换 xlsx -> csv
        :param data_filenamepath: xlsx 文件路径
        :return: csv 文件路径
        """
        def get_filename(path_filename):
            """
            获取文件所在文件夹路径、带拓展文件名、文件名、拓展名
            :param path_filename: 带拓展完整路径
            :return: 文件所在文件夹路径、带拓展文件名、文件名、拓展名
            """
            (filepath, tempfilename) = os.path.split(path_filename)
            (filename, extension) = os.path.splitext(tempfilename)
            return filepath, tempfilename, filename, extension
        filepath, tempfilename, filename, extension = get_filename(data_filenamepath)
        sheet = load_workbook(data_filenamepath).worksheets[0]
        rows = list(sheet.rows)
        print('********a')
        print(rows)
        sheet_val = []
        for row in rows:
            row_val = [col.value for col in row]
            sheet_val.append(row_val)
        print("excle表格转化csv", sheet_val[0])
        dt = pd.DataFrame(sheet_val[1:len(sheet_val)-1], columns=sheet_val[0])
        save_dir =''
        #if not os.path.exists(save_dir):
        #    os.makedirs(save_dir)
        CSVfilepath = os.path.join(save_dir, self.csv_name)
        print('aaa'+CSVfilepath)
        dt.to_csv(CSVfilepath, index=0)
        return CSVfilepath
if __name__ == '__main__':
    format_trans=format_translator()
    format_trans.XLSX2CSV('test_excel.xlsx')



import re
import csv
import os
import argparse
from .format_translator import format_translator

class GensystemRDL:
    '''
    classdocs
    '''
    rdl_file = ''
    offset_addr_str = 'Offset address'
    field_bit_str = 'Bits'
    Field='Field'
    Field_access='Field_access'
    Reset_value='Reset_value'
    Function='Function'

    def __init__(self):
        '''
        Constructor
        '''
        self._rgmItemList = []
        self.systemRDLFileName = ''
        self.regblock_name = 'test'
        self.regblock_description = 'test V2.0'
        self._properties = {'register':  '', 'address':  '', 'fields': []}

    def readRgmFile(self, filename):
        "read register file and update the internal database"
        rgmFile = open(filename, 'r')
        reader = csv.DictReader(rgmFile)

        for infos in reader:
            if (infos[self.offset_addr_str] != ''):
                self._properties['register'] = infos[self.field_bit_str]
                self._properties['address'] = infos[self.offset_addr_str]
                self._rgmItemList.append(self._properties.copy())
                self._properties = {'register':  '', 'address':  '', 'fields': []}
                fieldInfos = {}

            if (infos[self.offset_addr_str] == ''):
                fieldInfos['field'] = infos[self.Field]
                fieldInfos['field_access'] = infos[self.Field_access]
                fieldInfos['reset_value'] = infos[self.Reset_value]
                fieldInfos['bits'] = infos[self.field_bit_str]
                fieldInfos['function'] = infos[self.Function]

                self._rgmItemList[-1]['fields'].append(fieldInfos)
                fieldInfos = {}
      




    def gensystemRDLFile(self,filename1):
        with open(filename1, 'w') as systemRDLFile:
            def _writeaddrmap():
                self.rdl_file = self.rdl_file + ('addrmap %s \n' % self.regblock_name)
                self.rdl_file = self.rdl_file + ('{\n')
                self.rdl_file = self.rdl_file + ('   name = "%s";\n' % self.regblock_name)
                self.rdl_file = self.rdl_file + ('   desc = "%s";\n' %self.regblock_description)
                self.rdl_file = self.rdl_file + ('   default regwidth = 32;\n' )
                self.rdl_file = self.rdl_file + ('   default hw=rw;\n')
                systemRDLFile.write(self.rdl_file )

            def _writeRegClass(properties_index):
                systemRDLFile.write('   reg %s {\n' % properties_index['register'])
                systemRDLFile.write('       name="%s";\n' %properties_index['register'])
                for fd in properties_index['fields']:
                    systemRDLFile.write('       field{\n')
                    if (fd['field_access'] == 'RO'):
                        systemRDLFile.write('           sw=r;\n')
                       # systemRDLFile.write('           onread=r;\n')
                    elif (fd['field_access'] == 'RC'):
                        systemRDLFile.write('           sw=r;\n')
                        systemRDLFile.write('           onread=rclr;\n')
                    elif (fd['field_access'] == 'RS'):
                        systemRDLFile.write('           sw=r;\n')
                        systemRDLFile.write('           onread=rset;\n')
                    elif (fd['field_access'] == 'WO'):
                        systemRDLFile.write('           sw=w;\n')
                      #  systemRDLFile.write('           onwrite=w;\n')
                    elif (fd['field_access'] == 'WOC'):
                        systemRDLFile.write('           sw=w;\n')
                        systemRDLFile.write('           onwrite=wclr;\n')
                    elif (fd['field_access'] == 'WOS'):
                        systemRDLFile.write('           sw=w;\n')
                        systemRDLFile.write('           onwrite=wset;\n')
                    elif (fd['field_access'] == 'WO1'):
                        systemRDLFile.write('           sw=w1;\n')
                     #   systemRDLFile.write('           onwrite=w;\n')
                    elif (fd['field_access'] == 'RW'):
                        systemRDLFile.write('           sw=rw;\n')
                      #  systemRDLFile.write('           onread=r;\n')
                     #   systemRDLFile.write('           onwrite=w;\n')
                    elif (fd['field_access'] == 'W1C'):
                        systemRDLFile.write('           sw=rw;\n')
                      #  systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=woclr;\n')
                    elif (fd['field_access'] == 'W1S'):
                        systemRDLFile.write('           sw=rw;\n')
                    #    systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=woset;\n')
                    elif (fd['field_access'] == 'W1T'):
                        systemRDLFile.write('           sw=rw;\n')
                    #    systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wot;\n')
                    elif (fd['field_access'] == 'W0C'):
                        systemRDLFile.write('           sw=rw;\n')
                     #   systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wzc;\n')
                    elif (fd['field_access'] == 'W0S'):
                        systemRDLFile.write('           sw=rw;\n')
                     #   systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wzs;\n')
                    elif (fd['field_access'] == 'W0T'):
                        systemRDLFile.write('           sw=rw;\n')
                     #   systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wzt;\n')
                    elif (fd['field_access'] == 'WC'):
                        systemRDLFile.write('           sw=rw;\n')
                    #    systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wclr;\n')
                    elif (fd['field_access'] == 'WS'):
                        systemRDLFile.write('           sw=rw;\n')
                    #    systemRDLFile.write('           onread=r;\n')
                        systemRDLFile.write('           onwrite=wset;\n')
                    elif (fd['field_access'] == 'WRC'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rclr;\n')
                    #    systemRDLFile.write('           onwrite=w;\n')
                    elif (fd['field_access'] == 'W1SRC'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rclr;\n')
                        systemRDLFile.write('           onwrite=woset;\n')
                    elif (fd['field_access'] == 'W0SRC'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rclr;\n')
                        systemRDLFile.write('           onwrite=wzs;\n')
                    elif (fd['field_access'] == 'WSRC'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rclr;\n')
                        systemRDLFile.write('           onwrite=wset;\n')
                    elif (fd['field_access'] == 'WRS'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rset;\n')
                     #   systemRDLFile.write('           onwrite=w;\n')
                    elif (fd['field_access'] == 'W1CRS'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rset;\n')
                        systemRDLFile.write('           onwrite=woclr;\n')
                    elif (fd['field_access'] == 'W0CRS'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rset;\n')
                        systemRDLFile.write('           onwrite=wzc;\n')
                    elif (fd['field_access'] == 'WCRS'):
                        systemRDLFile.write('           sw=rw;\n')
                        systemRDLFile.write('           onread=rset;\n')
                        systemRDLFile.write('           onwrite=wclr;\n')
                    elif (fd['field_access'] == 'W1'):
                        systemRDLFile.write('           sw=rw1;\n')
                    #    systemRDLFile.write('           onread=r;\n')
                    #    systemRDLFile.write('           onwrite=w;\n')
                    systemRDLFile.write('           desc="%s";\n' % fd['function'])
                    systemRDLFile.write('       }%s[%s]=%s;\n' % (
                        fd['field'], fd['bits'], fd['reset_value']))
                systemRDLFile.write('   };\n')


            def _writeRgmend():

                systemRDLFile.write('};\n')
            # gensystemRDLFile main function below
            _writeaddrmap()
            for properties_index in self._rgmItemList:
                _writeRegClass(properties_index)
            for properties_index in self._rgmItemList:
                systemRDLFile.write(
                '%s %s @ %s;\n' % (properties_index['register'], properties_index['register'], properties_index['address']))
            _writeRgmend()

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-xlsx", help="display a path of csv file", type=str)
    parser.add_argument("-csv", help="display a path of csv file", type=str)
    args = parser.parse_args()
    format_trans=format_translator()
    if not os.path.exists(args.xlsx):
        print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
        print('ERROR')
        exit()
    print(args.xlsx)
  # if((args.csv==False)&&(args.xlsx)):
  #     print('%s \nXLSX TRANSFER TO RDL started \n%s\n' % (40 * '*', 40 * '*'))
  # if (args.csv == True && args.xlsx==False):
  #     print('%s \nCSV TRANSFER TO RDL started \n%s\n' % (40 * '*', 40 * '*'))
  # if (args.csv == False && args.xlsx==False):
  #     print('%s \n NO PARAMETER ARGS INPUT \n%s\n' % (40 * '*', 40 * '*'))
    if not os.path.exists(args.xlsx):
        print('%s \nEXCSL FILE PATH NOT EXISTS \n%s\n' % (40*'*', 40*'*'))
        print('ERROR')
        exit()
    print(args.xlsx)

    format_trans.XLSX2CSV(args.xlsx)
    print('%s \n transfer success \n%s' % (40*'*', 40*'*'))
    gen = GensystemRDL()
    gen.readRgmFile('save_csv'+format_trans.csv_name)
    gen.gensystemRDLFile(gen.systemRDLFileName)
    print('%s \ntransfer success \n%s' % (40*'*', 40*'*'))




import os
import setuptools

#from distutils.core import setup
with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

with open(os.path.join("peakrdl_format_trans", "__about__.py"), encoding='utf-8') as f:
    v_dict = {}
    exec(f.read(), v_dict)
    version = v_dict['__version__']

setuptools.setup(
    name='peakrdl_format_trans',
    version=version,
    author='jiangqingliu',
    author_email='jiangqingliu822@gmail.com',
    url='https://github.com/jiangqingliu88/peakrdl_format_trans',
    description=long_description,
    license='LICENSE',
    packages=['peakrdl_format_trans'],
    package_dir={'': '.'},
    include_package_data=True,
    entry_points = {
        "peakrdl.exporters": [
            'formattrans = peakrdl_format_trans.__peakrdl__:MyExporterDescriptor'
        ],
        "peakrdl.importers": [
            'formattrans = peakrdl_format_trans.__peakrdl__:MyImporterDescriptor'
        ]
    },
    install_requires=[
        # 'python>=3.6.0',
        # 'pandas>=0.10.0'
    ],
)
